@@ -31,7 +31,7 @@
 
 struct _GnomeBGSlideShowPrivate
 {
-        GFile *file;
+        char *filename;
 
         double start_time;
         double total_duration;
@@ -67,13 +67,16 @@ struct _FileSize
 
 enum {
         PROP_0,
-        PROP_FILE,
+        PROP_FILENAME,
         PROP_START_TIME,
         PROP_TOTAL_DURATION,
         PROP_HAS_MULTIPLE_SIZES,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GnomeBGSlideShow, gnome_bg_slide_show, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GnomeBGSlideShow, gnome_bg_slide_show, G_TYPE_OBJECT)
+#define GNOME_BG_SLIDE_SHOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o),\
+                                            GNOME_BG_TYPE_SLIDE_SHOW,\
+                                            GnomeBGSlideShowPrivate))
 
 static void
 gnome_bg_slide_show_set_property (GObject       *object,
@@ -89,8 +92,8 @@ gnome_bg_slide_show_set_property (GObject       *object,
 
         switch (property_id)
         {
-        case PROP_FILE:
-                self->priv->file = g_object_ref (g_value_get_object (value));
+        case PROP_FILENAME:
+                self->priv->filename = g_value_dup_string (value);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -112,9 +115,6 @@ gnome_bg_slide_show_get_property (GObject     *object,
 
         switch (property_id)
         {
-        case PROP_FILE:
-                g_value_set_object (value, self->priv->file);
-                break;
         case PROP_START_TIME:
                 g_value_set_int (value, self->priv->start_time);
                 break;
@@ -164,9 +164,8 @@ gnome_bg_slide_show_finalize (GObject *object)
 
         g_queue_free (self->priv->slides);
 
-        g_queue_free_full (self->priv->stack, g_free);
-
-        g_object_unref (self->priv->file);
+        g_list_foreach (self->priv->stack->head, (GFunc) g_free, NULL);
+        g_queue_free (self->priv->stack);
 }
 
 static void
@@ -181,11 +180,11 @@ gnome_bg_slide_show_class_init (GnomeBGSlideShowClass *self_class)
         gobject_class->finalize = gnome_bg_slide_show_finalize;
 
         g_object_class_install_property (gobject_class,
-                                         PROP_FILE,
-                                         g_param_spec_object ("file",
-                                                              "File",
-                                                              "File",
-                                                              G_TYPE_FILE,
+                                         PROP_FILENAME,
+                                         g_param_spec_string ("filename",
+                                                              "Filename",
+                                                              "Filename",
+                                                              NULL,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
         g_object_class_install_property (gobject_class,
@@ -211,12 +210,14 @@ gnome_bg_slide_show_class_init (GnomeBGSlideShowClass *self_class)
                                                                "Has multiple sizes",
                                                                FALSE,
                                                                G_PARAM_READABLE));
+
+        g_type_class_add_private (gobject_class, sizeof (GnomeBGSlideShowPrivate));
 }
 
 static void
 gnome_bg_slide_show_init (GnomeBGSlideShow *self)
 {
-        self->priv = gnome_bg_slide_show_get_instance_private (self);
+        self->priv = GNOME_BG_SLIDE_SHOW_GET_PRIVATE (self);
 
         self->priv->stack = g_queue_new ();
         self->priv->slides = g_queue_new ();
@@ -224,7 +225,7 @@ gnome_bg_slide_show_init (GnomeBGSlideShow *self)
 
 /**
  * gnome_bg_slide_show_new:
- * @filename: The filename of the slide show
+ * @filename: The name of the slide show file
  *
  * Creates a new object to manage a slide show.
  * window background between two #cairo_surface_ts.
@@ -234,17 +235,9 @@ gnome_bg_slide_show_init (GnomeBGSlideShow *self)
 GnomeBGSlideShow *
 gnome_bg_slide_show_new (const char *filename)
 {
-	GFile *file;
-	GnomeBGSlideShow *self;
-
-	file = g_file_new_for_path (filename);
-
-        self = GNOME_BG_SLIDE_SHOW (g_object_new (GNOME_BG_TYPE_SLIDE_SHOW,
-                                                  "file", file,
+        return GNOME_BG_SLIDE_SHOW (g_object_new (GNOME_BG_TYPE_SLIDE_SHOW,
+                                                  "filename", filename,
                                                   NULL));
-	g_object_unref (file);
-
-	return self;
 }
 
 static void
@@ -509,7 +502,9 @@ now (void)
  * @file1: (out) (allow-none) (transfer none): first file in slide
  * @file2: (out) (allow-none) (transfer none): second file in slide
  *
- * Returns the current slides progress.
+ * Returns the current slides progress
+ *
+ * Return value: %TRUE if successful
  **/
 void
 gnome_bg_slide_show_get_current_slide (GnomeBGSlideShow  *self,
@@ -543,7 +538,7 @@ gnome_bg_slide_show_get_current_slide (GnomeBGSlideShow  *self,
                         if (is_fixed)
                             *is_fixed = slide->fixed;
 
-                        if (file1 && slide->file1)
+                        if (file1)
                             *file1 = find_best_size (slide->file1, width, height);
 
                         if (file2 && slide->file2)
@@ -568,7 +563,6 @@ gnome_bg_slide_show_get_current_slide (GnomeBGSlideShow  *self,
  * @frame_number: frame number
  * @width: monitor width
  * @height: monitor height
- * @progress: (out) (allow-none): slide progress
  * @duration: (out) (allow-none): slide duration
  * @is_fixed: (out) (allow-none): if slide is fixed
  * @file1: (out) (allow-none) (transfer none): first file in slide
@@ -678,23 +672,19 @@ parse_file_contents (GnomeBGSlideShow  *self,
         g_markup_parse_context_free (context);
 
         if (!failed) {
-                guint queue_length;
+                int len;
 
                 t = mktime (&self->priv->start_tm);
 
                 self->priv->start_time = (double)t;
 
-                queue_length = g_queue_get_length (self->priv->slides);
+                len = g_queue_get_length (self->priv->slides);
 
                 /* no slides, that's not a slideshow */
-                if (queue_length == 0) {
-                        g_set_error_literal (error,
-                                             G_MARKUP_ERROR,
-                                             G_MARKUP_ERROR_INVALID_CONTENT,
-                                             "file is not a slide show since it has no slides");
+                if (len == 0) {
                         failed = TRUE;
                 /* one slide, there's no transition */
-                } else if (queue_length == 1) {
+                } else if (len == 1) {
                         Slide *slide = self->priv->slides->head->data;
                         slide->duration = self->priv->total_duration = G_MAXUINT;
                 }
@@ -716,14 +706,16 @@ gboolean
 gnome_bg_slide_show_load (GnomeBGSlideShow  *self,
                           GError           **error)
 {
+        GFile *file;
         char  *contents;
         gsize  length;
         gboolean parsed;
 
-        if (!g_file_load_contents (self->priv->file, NULL, &contents, &length,
-                                   NULL, NULL)) {
+        file = g_file_new_for_path (self->priv->filename);
+        if (!g_file_load_contents (file, NULL, &contents, &length, NULL, NULL)) {
                 return FALSE;
         }
+        g_object_unref (file);
 
         parsed = parse_file_contents (self, contents, length, error);
         g_free (contents);
@@ -745,20 +737,17 @@ on_file_loaded (GFile        *file,
 
     if (!loaded) {
             g_task_return_error (task, error);
-            g_object_unref (task);
             return;
     }
 
     if (!parse_file_contents (g_task_get_source_object (task), contents, length, &error)) {
             g_task_return_error (task, error);
-            g_object_unref (task);
             g_free (contents);
             return;
     }
     g_free (contents);
 
     g_task_return_boolean (task, TRUE);
-    g_object_unref (task);
 }
 
 /**
@@ -777,11 +766,12 @@ gnome_bg_slide_show_load_async (GnomeBGSlideShow    *self,
                                 gpointer             user_data)
 {
     GTask *task;
+    GFile *file;
 
     task = g_task_new (self, cancellable, callback, user_data);
 
-    g_file_load_contents_async (self->priv->file, cancellable,
-                                (GAsyncReadyCallback) on_file_loaded, task);
+    file = g_file_new_for_path (self->priv->filename);
+    g_file_load_contents_async (file, cancellable, (GAsyncReadyCallback) on_file_loaded, task);
 }
 
 /**
